package reborncore.common.explosion;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.init.Blocks;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.Explosion;
import net.minecraft.world.World;

import org.apache.commons.lang3.time.StopWatch;

import reborncore.RebornCore;

/**
 * Created by modmuss50 on 12/03/2016.
 */
public class RebornExplosion extends Explosion
{

	@Nonnull
	BlockPos center;

	@Nonnull
	World world;

	@Nonnull
	int radius;

	@Nullable
	EntityLivingBase livingBase;

	public RebornExplosion(@Nonnull BlockPos center, @Nonnull World world, @Nonnull int radius)
	{
		super(world, null, center.func_177958_n(), center.func_177956_o(), center.func_177952_p(), radius, false, true);
		this.center = center;
		this.world = world;
		this.radius = radius;
	}

	public void setLivingBase(@Nullable EntityLivingBase livingBase)
	{
		this.livingBase = livingBase;
	}

	public @Nullable EntityLivingBase getLivingBase()
	{
		return livingBase;
	}

	public void explode()
	{
		StopWatch watch = new StopWatch();
		watch.start();
		for (int tx = -radius; tx < radius + 1; tx++)
		{
			for (int ty = -radius; ty < radius + 1; ty++)
			{
				for (int tz = -radius; tz < radius + 1; tz++)
				{
					if (Math.sqrt(Math.pow(tx, 2) + Math.pow(ty, 2) + Math.pow(tz, 2)) <= radius - 2)
					{
						BlockPos pos = center.func_177982_a(tx, ty, tz);
						IBlockState state = world.func_180495_p(pos);
						Block block = state.func_177230_c();
						if (block != Blocks.field_150357_h && block != Blocks.field_150350_a)
						{
							block.func_180652_a(world, pos, this);
							world.func_180501_a(pos, Blocks.field_150350_a.func_176223_P(), 3);
						}
					}
				}
			}
		}
		RebornCore.logHelper.info("The explosion took" + watch + " to explode");
	}

	@Override
	public void func_77278_a()
	{
		explode();
	}

	@Override
	public void func_77279_a(boolean spawnParticles)
	{
		explode();
	}

	@Override
	public @Nullable EntityLivingBase func_94613_c()
	{
		return livingBase;
	}

	@Override
	public List<BlockPos> func_180343_e()
	{
		List<BlockPos> poses = new ArrayList<>();
		for (int tx = -radius; tx < radius + 1; tx++)
		{
			for (int ty = -radius; ty < radius + 1; ty++)
			{
				for (int tz = -radius; tz < radius + 1; tz++)
				{
					if (Math.sqrt(Math.pow(tx, 2) + Math.pow(ty, 2) + Math.pow(tz, 2)) <= radius - 2)
					{
						BlockPos pos = center.func_177982_a(tx, ty, tz);
						IBlockState state = world.func_180495_p(pos);
						Block block = state.func_177230_c();
						if (block != Blocks.field_150357_h && block != Blocks.field_150350_a)
						{
							poses.add(pos);
						}
					}
				}
			}
		}
		return poses;
	}
}
